/*
 * Copyright (C) 2003 Ranjan Ghosh <ghosh@linux>
 */

#include "regimp3view.h"

#include <qpainter.h>
#include <qlayout.h>

#include <kurl.h>

#include <iostream>
using namespace std;

#include <ktrader.h>
#include <kapplication.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include <kwin.h>
#include <netwm.h>
#include <X11/Xlib.h>

bool isWindowOfProcess(Window w, const char* name) {
  KWin::Info wininfo;
  wininfo=KWin::info(w);
  if (wininfo.mappingState!=NET::Withdrawn) {
    QString procpath,pid;
    procpath="/proc/";
    pid.setNum(wininfo.pid);
    procpath+=pid;
    procpath+="/exe";
    char buf[240];
    int ccount=readlink(procpath.latin1(),buf,240-1);
    if (ccount>0) {
      buf[ccount]=0; //Zero isn't appended automatically
      if (strstr(buf,name)) return true;
    }
  }
  return false;
}

int enumWindows(Display* display,const char* name,Window enumarray[]) {
  int windowcount=0;
  Window w;
  NETRootInfo* rootinfo=new NETRootInfo(display,NET::ClientList);
  for (int j=0;j<rootinfo->clientListCount();j++) {
    w=(rootinfo->clientList())[j];
    if (!isWindowOfProcess(w,name)) continue;

    enumarray[windowcount]=w;
    windowcount++;
  }
  return windowcount;
}

regimp3View::regimp3View(QWidget *parent)
    : QWidget(parent),
      DCOPObject("regimp3Iface")
{

 canShutdown=false;
 showMaximized();

	setFocusPolicy(QWidget::ClickFocus);

	//Display* display;
 //display=XOpenDisplay(NULL);

 system("gimp-1.3 &");

	//XCloseDisplay(display);

 gimprunning=false;

 QTimer *timer=new QTimer(this);
 connect(timer,SIGNAL(timeout()),this,SLOT(timerDone()));
 timer->start(500,false);
}

regimp3View::~regimp3View()
{
}

void regimp3View::closeGIMP() {
 int windowcount;
 Window windowlist[200];
 Window mainwin=0;

	Display* display;
 display=XOpenDisplay(NULL);

 windowcount=enumWindows(display,"gimp-1.3",windowlist);
 for (int i=0;i<windowcount;i++) {
   KWin::Info info=KWin::info(windowlist[i]);
   if (info.name=="GIMP") mainwin=windowlist[i];
 }

 if (mainwin!=0) {
   NETRootInfo* rootinfo=new NETRootInfo(display,NET::CloseWindow);
   rootinfo->closeWindowRequest(mainwin);
   }

 XCloseDisplay(display);
}

void regimp3View::timerDone() {
 int windowcount;
 Window windowlist[200];

	Display* display;
 display=XOpenDisplay(NULL);

 //Remove all GIMP-Windows from taskbar and put always on top
	windowcount=enumWindows(display,"regimp3",windowlist);
 for (int i=0;i<windowcount;i++) {
   if (windowlist[i]==parentWidget()->winId()) continue;
   KWin::setState(windowlist[i],NET::StaysOnTop);
 }

 //Remove all GIMP-Windows from taskbar and put always on top
	windowcount=enumWindows(display,"gimp-1.3",windowlist);
 for (int i=0;i<windowcount;i++) {
   KWin::setState(windowlist[i],NET::SkipTaskbar);
   KWin::setState(windowlist[i],NET::StaysOnTop);
 }

 XCloseDisplay(display);
 if ((windowcount==0) and (gimprunning)) {
   canShutdown=true;
   kapp->quit();
   }
 if ((windowcount>1) and (!gimprunning)) gimprunning=true;
}

void regimp3View::hideEvent(QHideEvent *e) {
 int windowcount;
 Window windowlist[200];

	Display* display;
 Window w;
 display=XOpenDisplay(NULL);

	windowcount=enumWindows(display,"gimp-1.3",windowlist);
 for (int i=0;i<windowcount;i++) {
	  w=windowlist[i];
   KWin::clearState(w,NET::SkipTaskbar);
   KWin::iconifyWindow(w,false);
   KWin::setState(w,NET::SkipTaskbar);
 }

	XCloseDisplay(display);
}

void regimp3View::activateGIMP() {
 int windowcount;
 Window windowlist[200];

 Display* display;
 display=XOpenDisplay(NULL);
	Window w;
 KWin::Info wininfo;

	windowcount=enumWindows(display,"gimp-1.3",windowlist);
	for (int i=0;i<windowcount;i++) {
	  w=windowlist[i];
   KWin::setState(w,NET::SkipTaskbar);
   KWin::deIconifyWindow(w,false);
   KWin::setActiveWindow(w);
	}

	XCloseDisplay(display);
}

void regimp3View::focusInEvent(QFocusEvent *e) {
}

void regimp3View::showEvent(QShowEvent *e) {
  activateGIMP();
}

#include "regimp3view.moc"
